// Copyright (c) 2013, Sandro Andreotti and Stefan Canzar,
// FU Berlin and Johns Hopkins University School of Medicine, Baltimore
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the authors or the contributing institutions nor
//       the names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL ANY OF THE AUTHORS OR THE CONTRIBUTING
// INSTITUTIONS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
// OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
// IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#ifndef EVENTHOLDERBASE_H
#define EVENTHOLDERBASE_H

#include <Types.h>
#include <seqan/index.h>

struct EventHolderBase
{
protected:  

  typedef seqan::Index<GeneString> TIndex;
  typedef seqan::String<seqan::Size<TIndex>::Type> TSuffArray;
  typedef seqan::String<seqan::SAValue<TIndex>::Type> TLcpArray;

  void _createEventsLCP(const GeneString & str, size_t catPos);
  virtual void _reportEvent(size_t startPos, size_t endPos, size_t len, const TSuffArray &sa, size_t catPos) = 0;
};


inline void EventHolderBase::_createEventsLCP(const GeneString & str, size_t catPos)
{
  using std::vector;
  typedef seqan::Index<GeneString> TIndex;
  TIndex myIndex(str);
  seqan::resize(seqan::indexSA(myIndex), seqan::length(myIndex));
  seqan::resize(seqan::indexLcp(myIndex), seqan::length(myIndex));
  seqan::createSuffixArray(seqan::indexSA(myIndex), seqan::indexText(myIndex), seqan::SAQSort());
  seqan::indexCreate(myIndex, seqan::EsaLcp());

  seqan::String<seqan::Size<TIndex>::Type> lcp = seqan::indexLcp(myIndex);
  seqan::String<seqan::SAValue<TIndex>::Type> sa = seqan::indexSA(myIndex);

  size_t lastLCP = 0;
  std::vector<size_t>startPos(1000,0);
  for (size_t i = 0; i < seqan::length(str); ++i)
  {
#ifdef DEBUG
    std::cerr << i << "  " << lcp[i] << "  " << sa[i] << std::endl;
#endif
    if (lastLCP < lcp[i])
    {
      //set new starting iterators
      for (size_t l = lastLCP + 1; l <= lcp[i]; ++l)
      {
        startPos[l] = i;
      }
    }
    else if (lastLCP > lcp[i])
    {
      for (size_t l = lastLCP; l > lcp[i]; --l)
      {
        if(l > 0)
          _reportEvent(startPos[l], i, l, sa, catPos); //call derived class member
      }
    }
    lastLCP = lcp[i];
  }
}

#endif // EVENTHOLDERBASE_H
