// Copyright (c) 2013, Sandro Andreotti and Stefan Canzar,
// FU Berlin and Johns Hopkins University School of Medicine, Baltimore
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the authors or the contributing institutions nor
//       the names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL ANY OF THE AUTHORS OR THE CONTRIBUTING
// INSTITUTIONS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
// OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
// IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#ifndef ILPMEDIAN_H
#define ILPMEDIAN_H

#include <Types.h>
#include <SparsePairGraph.h>
#include <DuplicationEventHolder.h>
#include <PairedDuplicationEventHolder3.h>
#include <InversionEventHolder.h>
#include <MixedCycleFinder.h>


struct column; //forward declaration
typedef std::vector<column> TAlignment;

//CALLBACK STUFF

struct IlpOptions
{
    bool targetDisruptConstraints;
    bool sourceDisruptConstraints;
    size_t maxNumNonImprovedRounds;
    double maxBestCurrentNodeDelta;
    bool liftedDupCycles;
    bool addOnlyBestDupCycle;
    bool inversions;
    bool asymmetric;
    
    IlpOptions() : targetDisruptConstraints(true),
    sourceDisruptConstraints(true),
    maxNumNonImprovedRounds(10),
    maxBestCurrentNodeDelta(4),
    liftedDupCycles(true),
    addOnlyBestDupCycle(true),
    inversions(false), //not yet supported
    asymmetric(true) //only this is supported now
    {}
    
};

struct CallBackDataMedian2
{
    CallBackDataMedian2(){;}
    CallBackDataMedian2(const CallBackDataMedian2 &in):
    opt(in.opt),
    pairGraphs(in.pairGraphs),
    pairGraphsAug(in.pairGraphsAug),
    dupEvents(in.dupEvents),
    pairedDupEvents(in.pairedDupEvents),
    mcFinder(in.mcFinder)
    {
        std::cerr << "In Copy Constr!!" << std::endl;
    }
    
    IlpOptions opt;
    MixedCycleFinder::TPairGraphMatrix  pairGraphs;
    MixedCycleFinder::TPairGraphMatrix  pairGraphsAug;
    std::vector<DuplicationEventHolder> dupEvents;
    std::vector <PairedDuplicationEventHolder3> pairedDupEvents;
    MixedCycleFinder mcFinder;
    
    double lastImprovedScore;
    size_t lastImproveRound;
};

struct VarHolderMedian2
{
    std::vector<std::vector<IloNumVarArray> > alignEdgesVars;
    std::vector<std::vector<IloNumVarArray> > inversionVars;
    std::vector<IloNumVarArray> duplicationVars;
    std::vector<std::vector<IloNumVarArray> > lossVars;
    std::vector<IloNumVarArray> pairedDuplicationVars;
    std::vector<IloNumVarArray> wVars;
    std::vector<IloNumVarArray> penaltyVars;
};

struct ValHolderMedian2
{
    std::vector<std::vector<IloNumArray> > alignEdgesVals;
    std::vector<std::vector<IloNumArray> > inversionVals;
    std::vector<IloNumArray> duplicationVals;
    std::vector<std::vector<IloNumArray> > lossVals;
    std::vector<IloNumArray> pairedDuplicationVals;
    std::vector<IloNumArray> wVals;
    std::vector<IloNumArray> penaltyVals;
};

double MedianILPThreeLeaf(const std::vector<GeneString> & genomes, TAlignment & align, const IlpOptions & opts);


//POSTPROCESSING ALIGNMENT STUFF
struct column
{
    enum Type {DUP, PAIRED, AL, LOSS, LOSSCONS, INV, DUPLOSS, UNDEF};
    Type type;
    std::vector<size_t> alPos;
    std::pair<size_t, size_t> id; //id for loss or duplication (pair of sequence and event id)
    TAlphabet consensus; //the consensus symbol of the column;
    
    bool operator< (const column & rhs) const
    {
        size_t i = 0;
        while(i < alPos.size() && i < rhs.alPos.size())
        {
            if (alPos[i] != rhs.alPos[i] && alPos[i] != -1 && rhs.alPos[i] != -1)
                return alPos[i] < rhs.alPos[i];
            ++i;
        }
        return false;
    }
};


void getMedianGenome(const TAlignment & align, GeneString & median);
void getDistancesToMedian(const TAlignment & align, std::vector<double> & dist, bool asymmetric = false);





#endif
