// Copyright (c) 2013, Sandro Andreotti and Stefan Canzar,
// FU Berlin and Johns Hopkins University School of Medicine, Baltimore
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the authors or the contributing institutions nor
//       the names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL ANY OF THE AUTHORS OR THE CONTRIBUTING
// INSTITUTIONS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
// OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
// IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#ifndef MIXEDCYCLEFINDER_H
#define MIXEDCYCLEFINDER_H

#include <SparsePairGraph.h>

struct MixedCycleFinder
{

    struct TVertexInfo
    {
        //Constructor
        TVertexInfo(size_t s, size_t p): seq(s), pos(p){}
        TVertexInfo(): seq(-1), pos(-1){}
        size_t seq; //the sequence number
        size_t pos;  //the position
    };

    struct TEdgeInfo
    {
        size_t seqS, seqT, posS, posT;
    };

    typedef TDirGraph TGraph;
    typedef std::vector<std::vector<SparsePairGraph> > TPairGraphMatrix;
    typedef std::vector<SparsePairGraph::TVertexList> TAlignGraphVertices;
    typedef seqan::String<TVertexInfo> TVertexInfoMap;
    typedef std::vector<std::vector<IloNumArray> > TWeightMatrix;
    typedef seqan::String< seqan::String<SparsePairGraph::TPredMap> > TPredMapMatrix;


    struct TMixedCycleIndex
    {
        TMixedCycleIndex(size_t a,
                         size_t b,
                         size_t i,
                         size_t pa = -1,
                         size_t pb = -1) : seqA(a), seqB(b), posA(pa), posB(pb), index(i){}
        size_t seqA;
        size_t seqB;
        size_t posA;
        size_t posB;
        size_t index;
    };

    typedef std::vector<TMixedCycleIndex> TMixedCycle;

protected:



//    struct TEdgeInfo
//    {
//        //Constructor
//        //TVertexInfo(size_t s, size_t p): seq(s), pos(p){}
//        size_t seqA; //the sequence number
//        size_t seqB; //the sequence number
//        TSizeArray ids;  //the ids of the corresponding edges in the pg of seqA and seqB
//    };

    //the sparse pairgraphs
    TPairGraphMatrix _pGs;

    //The (extended) alignment graph
    TDirGraph _alnGraph;

    //accessors to the vertices of each sequence in the alignment graph
    TAlignGraphVertices _vMat;

    TVertexInfoMap _alnGraphToPG;
    TDistanceMap _edgeWeights;

    //The predecessor maps of all shortest path trees
    TPredMapMatrix _predMaps;

    //helper
    void _buildAlignGraph();
    void _prepareAlnGraph(const TWeightMatrix & weights);

public:    
    void findViolatedMixedCycle(const TWeightMatrix &alnScores,
                                TMixedCycle &cycle,
                                size_t & length,
                                IloNum eps);

    MixedCycleFinder (const TPairGraphMatrix & pairGraphs);
    MixedCycleFinder (){};
};


#endif // MIXEDCYCLEFINDER_H


