// Copyright (c) 2013, Sandro Andreotti and Stefan Canzar,
// FU Berlin and Johns Hopkins University School of Medicine, Baltimore
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the authors or the contributing institutions nor
//       the names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL ANY OF THE AUTHORS OR THE CONTRIBUTING
// INSTITUTIONS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
// OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
// IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#ifndef PAIREDDUPLICATIONEVENTHOLDER3_H
#define PAIREDDUPLICATIONEVENTHOLDER3_H

#include <Types.h>
#include <DuplicationEventHolderBase.h>

struct PairedDuplicationEventHolder3 : public DuplicationEventHolderBase
{
    friend struct SparsePairGraph;    
    typedef std::vector<std::vector<size_t> > TDuplicationMap;
    typedef std::vector<DuplicationEvent> TDuplicationEventList;
    
    //MEMBERS    
    ///find all possible duplications using lcp tab scan
    void createDuplicationEventsLCP(const GeneString & strP, const GeneString & strC, bool inversions = false);
    void createDuplicationEventsLCP(const GeneString & strP, bool inversions = false)
    {
        std::cerr << "NOT IMPLEMENTED! " << std::endl;
    }    
    
    void _reportEvent(size_t startPos, size_t endPos, size_t len, const TSuffArray &sa, size_t catPos);
    
    void combineScores(const DuplicationEventHolderBase & holder/*, bool init*/);
    TDuplicationMap dupMapS;
    
protected:


};


inline void PairedDuplicationEventHolder3::_reportEvent(size_t startPos, size_t endPos, size_t len, const TSuffArray &sa, size_t catPos)
{ 
    size_t len2 = len - 1;
    size_t lenG2 = seqan::length(sa) - catPos - 1;
    
    for (size_t i = startPos; i < endPos; ++i)
    {
        for (size_t j = i + 1; j <= endPos; ++j)
        {
            size_t l = seqan::_min(sa[i],sa[j]);
            size_t r = seqan::_max(sa[i],sa[j]);
            
            //std::cerr << l << "  " << r << "  " << catPos << std::endl;
            
            if (l > catPos || r < catPos)
                continue; //skip if both on same side of concat
            
            if (_inversions)
                r = lenG2 - (r - catPos) - len2; // for inverted duplication perform index correction
            else
                r = r - catPos - 1;
            
            duplications.push_back(DuplicationEvent(l, l + len2, r, r + len2));
            
            for(size_t k = 0; k <= len2; ++k)
                dupMap[r + k].push_back(seqan::length(duplications)-1);
            
            dupInterMap[r][r + len2].push_back(seqan::length(duplications)-1);
        }
    }
}




#endif
