#ifndef SUBTOUR_H
#define SUBTOUR_H

#include <ilcplex/ilocplex.h>

#include <SparsePairGraph.h>
#include <DuplicationEventHolder.h>

#include <lemon/list_graph.h>
#include <lemon/concepts/maps.h>

using namespace lemon;

struct ViolatedCut {
  double cutvalue;
  std::vector<int> node_idc;

  bool operator<(const ViolatedCut& rhs) const
  {
    for(int i=0; i<std::min(node_idc.size(), rhs.node_idc.size()); i++)
      {
	if( node_idc[i] < rhs.node_idc[i] )
	  return true;
	else if( node_idc[i] > rhs.node_idc[i] )
	  return false;	
      }
    if( node_idc.size() < rhs.node_idc.size() )
      return true;
    else 
      return false;
  }
};

void _get_violated_cuts(ListDigraph& g, 
			ArcLookUp<ListDigraph> & ae,
			ListDigraph::Node* nodes,
			ListDigraph::ArcMap<double>& cap,
			int s_idx,
			int length_string, 
			IloNum eps,
			std::set<ViolatedCut>& cutsx);


void _get_reverse_violated_cuts(ListDigraph& g, 
				  ListDigraph::Node* nodes, 
				  int s_idx,
				  ArcLookUp<ListDigraph>& ae,
				  ListDigraph::ArcMap<double>& cap,
				  ListDigraph::ArcMap<double>& dupcap, 
				  ListDigraph::ArcMap<bool>& is_duparc, 
				  ListDigraph::NodeMap<double>& sum_lossalign,
				  int length_string,
				  std::set<ViolatedCut>& cuts,
				  IloNum eps);


void _create_flowgraph_s(ListDigraph & g,
			 ListDigraph::Node* nodes,
			 ArcLookUp<ListDigraph> & ae,
			 ListDigraph::ArcMap<double> & cap, 
			 ListDigraph::ArcMap<double> & dupcap,
			 ListDigraph::ArcMap<bool> & is_duparc,
			 int s_idx,
			 DuplicationEventHolder& dupEvents,
			 const IloNumArray &duplicationsVal,
			 const IloNumArray &lossVal, 
			 const IloNumArray &alignEdgesVal,
			 SparsePairGraph & pairGraph,
			 ListDigraph::NodeMap<double> & sum_lossalign,			 
			 int length_otherstring, 
			 IloNum eps,
			 bool stringA);

void get_violated_cuts(DuplicationEventHolder& dupEvents,
		       const IloNumArray& duplicationsVal,
		       const IloNumArray& lossVal,
		       const IloNumArray& alignEdgesVal, 
		       SparsePairGraph& pairGraph, 
		       int length_otherstring, 
		       IloNum eps,
		       ListDigraph& g,
		       ListDigraph::Node* nodes,
		       std::set<ViolatedCut>& cuts, 
		       bool stringA);		


#endif
