// Copyright (c) 2013, Sandro Andreotti and Stefan Canzar,
// FU Berlin and Johns Hopkins University School of Medicine, Baltimore
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the authors or the contributing institutions nor
//       the names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL ANY OF THE AUTHORS OR THE CONTRIBUTING
// INSTITUTIONS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
// OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
// IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <DuplicationEventHolderBase.h>

void DuplicationEventHolderBase::DPCompute(const IloNumArray & scores)
{
    //DP to compute the pi and sigma entries used for mixed clique detection
    size_t lenS = dupInterMap.size();
    //std::cerr << dupInterMap.size() << "  vs. " << duplications.size() << std::endl;
    
    for (size_t p = 0; p < lenS; ++p)
    {
        double score = 0.;
        for (size_t i = 0; i < dupInterMap[p][lenS - 1].size(); ++i)
        {
            score += scores[dupInterMap[p][lenS - 1][i]];
        }
        
        piV[p][lenS - 1] = score;
        for(size_t q = lenS - 2; (int)q >= (int)p; --q)
        {
            //std::cerr << "Q: " << q <<std::endl;
            score = 0.;
            assert(p < dupInterMap.size());
            for (size_t i = 0; i < dupInterMap[p][q].size(); ++i)
            {
                //std::cerr << p << "  " << q << " " << i << "  ::  " << dupInterMap[p][q][i] << std::endl;
                score += scores[dupInterMap[p][q][i]];
            }
            piV[p][q] = piV[p][q + 1] + score;
        }
    }
    
    for (size_t q = 0; q < lenS; ++q)
    {
        sigV[0][q] = piV[0][q];
    }
    
    for (size_t p = 1; p < lenS; ++p)
    {
        for (size_t q = p; q < lenS; ++q)
        {
            sigV[p][q] = sigV[p - 1][q] + piV[p][q];
        }
    }
    
    for (size_t p = 0; p < lenS; ++p)
    {
        double score = 0.;
        for (size_t i = 0; i < dupInterMap[0][p].size(); ++i)
        {
            score += scores[dupInterMap[0][p][i]];
        }
        
        pirV[0][p] = score;
        for(size_t q = 1; (int)q <= (int)p; ++q)
        {
            //std::cerr << "Q: " << q <<std::endl;
            score = 0.;
            for (size_t i = 0; i < dupInterMap[q][p].size(); ++i)
            {
                //std::cerr << p << "  " << q << " " << i << "  ::  " << dupInterMap[p][q][i] << std::endl;
                score += scores[dupInterMap[q][p][i]];
            }
            pirV[q][p] = pirV[q - 1][p] + score;
        }
    }
}

void DuplicationEventHolderBase::_init()
{
    size_t lenS = dupInterMap.size();
    
    for (size_t p = 0; p < lenS; ++p)
    {
        piI[p][lenS - 1] = dupInterMap[p][lenS - 1];
        for(size_t q = lenS - 2; (int)q >= (int)p; --q)
        {
            piI[p][q] = piI[p][q + 1];
            piI[p][q].insert(piI[p][q].end(), dupInterMap[p][q].begin(), dupInterMap[p][q].end());
        }
    }
    
    for (size_t q = 0; q < lenS; ++q)
    {
        sigI[0][q] = piI[0][q];
    }
    
    for (size_t p = 1; p < lenS; ++p)
    {
        for (size_t q = p; q < lenS; ++q)
        {
            sigI[p][q] = sigI[p - 1][q];
            sigI[p][q].insert(sigI[p][q].end(), piI[p][q].begin(), piI[p][q].end());
        }
    }
}

