// Copyright (c) 2013, Sandro Andreotti and Stefan Canzar,
// FU Berlin and Johns Hopkins University School of Medicine, Baltimore
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the authors or the contributing institutions nor
//       the names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL ANY OF THE AUTHORS OR THE CONTRIBUTING
// INSTITUTIONS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
// OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
// IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <PairedDuplicationEventHolder3.h>


void PairedDuplicationEventHolder3::createDuplicationEventsLCP(const GeneString & strS, const GeneString & strT, bool inversions)
{   
    _inversions = inversions;
    
    dupMap.clear();
    dupMap.resize(seqan::length(strT));
    
    dupMapS.clear();
    dupMapS.resize(seqan::length(strS));
    
    dupInterMap.assign(seqan::length(strT), std::vector<std::vector<size_t> >(seqan::length(strT)));
    duplications.clear();
    
    piV.assign(seqan::length(strT),  TFloatArray(seqan::length(strT), 0.));
    pirV.assign(seqan::length(strT),  TFloatArray(seqan::length(strT), 0.));
    sigV.assign(seqan::length(strT), TFloatArray(seqan::length(strT), 0.));
    piI.assign(seqan::length(strT), std::vector<TSizeArray>(seqan::length(strT)));
    sigI.assign(seqan::length(strT), std::vector<TSizeArray>(seqan::length(strT)));    
    
    size_t catPos = seqan::length(strS);
    if (_inversions)
    {
        GeneString strTmp = strS;
        GeneString inv = strT;
        seqan::appendValue(strTmp,0); //splitter
        seqan::reverse(inv);
        for (int i = 0; i < seqan::length(inv); ++i)
            inv[i] *= -1;
        
        seqan::append(strTmp, inv);
        //std::cerr << seqan::length(strTmp) << "  "  << catPos << std::endl;
        _createEventsLCP(strTmp, catPos);
        _inversions = false;
    }
    
    GeneString strTmp = strS;
    seqan::appendValue(strTmp,0); //splitter
    seqan::append(strTmp, strT);
    //std::cerr << seqan::length(strTmp) << "  "  << catPos << std::endl;
    _createEventsLCP(strTmp, catPos);
    
    _init();
    
    for (size_t i = 0; i < duplications.size(); ++i)
    {
        for (size_t pos = duplications[i].sourceBegin; pos <= duplications[i].sourceEnd; ++pos)
        {
            dupMapS[pos].push_back(i);
        }
    }
}



void PairedDuplicationEventHolder3::combineScores(const DuplicationEventHolderBase & holder/*, bool init*/)
{
    for (size_t i = 0; i < piV.size(); ++i)
    {
        for(size_t j = 0; j < piV.size(); ++j)
        {
            piV[i][j] += holder.piV[i][j];
            pirV[i][j] += holder.pirV[i][j];
            sigV[i][j] += holder.sigV[i][j];         
        }
    }
}
